##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Password Utility Interfaces
"""
import zope.interface
import zope.schema
from zope.i18n import translate

from z3c.password import MessageFactory as _

class InvalidPassword(zope.schema.ValidationError):
    """Invalid Password"""

    i18n_message = None

    def __str__(self):
        if self.i18n_message:
            return translate(self.i18n_message)
        return super(InvalidPassword, self).__str__()

    def doc(self):
        if self.i18n_message:
            return self.i18n_message
        return self.__class__.__doc__


class NoPassword(InvalidPassword):
    __doc__ = _('''No new password specified.''')

class TooShortPassword(InvalidPassword):
    __doc__ = _('''Password is too short.''')

    def __init__(self, minLength=None):
        super(TooShortPassword, self).__init__()
        self.minLength = minLength
        if minLength is not None:
            self.i18n_message = _(
                'Password is too short (minimum length: ${minLength}).',
                mapping=dict(minLength=minLength))

class TooLongPassword(InvalidPassword):
    __doc__ = _('''Password is too long.''')

    def __init__(self, maxLength=None):
        super(TooLongPassword, self).__init__()
        self.maxLength = maxLength
        if maxLength is not None:
            self.i18n_message = _(
                'Password is too long (maximum length: ${maxLength}).',
                mapping=dict(maxLength=maxLength))

class TooSimilarPassword(InvalidPassword):
    __doc__ = _('''Password is too similar to old one.''')

    def __init__(self, similarity=None, maxSimilarity=None):
        super(TooSimilarPassword, self).__init__()
        self.similarity = similarity
        self.maxSimilarity = maxSimilarity
        if similarity is not None and maxSimilarity is not None:
            self.i18n_message = _(
                'Password is too similar to old one'
                ' (similarity ${similarity}%, should be at most ${maxSimilarity}%).',
                mapping=dict(similarity=int(round(similarity * 100)),
                             maxSimilarity=int(maxSimilarity * 100)))

class TooManyGroupCharacters(InvalidPassword):
    __doc__ = _('''Password contains too many characters of one group.''')

    def __init__(self, groupMax=None):
        super(TooManyGroupCharacters, self).__init__()
        self.groupMax = groupMax
        if groupMax is not None:
            self.i18n_message = _(
                'Password contains too many characters of one group'
                ' (should have at most ${groupMax}).',
                mapping=dict(groupMax=groupMax))

class TooFewGroupCharacters(InvalidPassword):
    __doc__ = _('''Password does not contain enough characters of one group.''')

class TooFewGroupCharactersLowerLetter(TooFewGroupCharacters):
    __doc__ = _(
        '''Password does not contain enough characters of lowercase letters.''')

    def __init__(self, minLowerLetter=None):
        super(TooFewGroupCharactersLowerLetter, self).__init__()
        self.minLowerLetter = minLowerLetter
        if minLowerLetter is not None:
            self.i18n_message = _(
                'Password does not contain enough characters of lowercase letters'
                ' (should have at least ${minLowerLetter}).',
                mapping=dict(minLowerLetter=minLowerLetter))

class TooFewGroupCharactersUpperLetter(TooFewGroupCharacters):
    __doc__ = _(
        '''Password does not contain enough characters of uppercase letters.''')

    def __init__(self, minUpperLetter=None):
        super(TooFewGroupCharactersUpperLetter, self).__init__()
        self.minUpperLetter = minUpperLetter
        if minUpperLetter is not None:
            self.i18n_message = _(
                'Password does not contain enough characters of uppercase letters'
                ' (should have at least ${minUpperLetter}).',
                mapping=dict(minUpperLetter=minUpperLetter))

class TooFewGroupCharactersDigits(TooFewGroupCharacters):
    __doc__ = _('''Password does not contain enough characters of digits.''')

    def __init__(self, minDigits=None):
        super(TooFewGroupCharactersDigits, self).__init__()
        self.minDigits = minDigits
        if minDigits is not None:
            self.i18n_message = _(
                'Password does not contain enough characters of digits'
                ' (should have at least ${minDigits}).',
                mapping=dict(minDigits=minDigits))

class TooFewGroupCharactersSpecials(TooFewGroupCharacters):
    __doc__ = _(
        '''Password does not contain enough characters of special characters.''')

    def __init__(self, minSpecials=None):
        super(TooFewGroupCharactersSpecials, self).__init__()
        self.minSpecials = minSpecials
        if minSpecials is not None:
            self.i18n_message = _(
                'Password does not contain enough characters of special characters'
                ' (should have at least ${minSpecials}).',
                mapping=dict(minSpecials=minSpecials))

class TooFewGroupCharactersOthers(TooFewGroupCharacters):
    __doc__ = _(
        '''Password does not contain enough characters of other characters.''')

    def __init__(self, minOthers=None):
        super(TooFewGroupCharactersOthers, self).__init__()
        self.minOthers = minOthers
        if minOthers is not None:
            self.i18n_message = _(
                'Password does not contain enough characters of other characters'
                ' (should have at least ${minOthers}).',
                mapping=dict(minOthers=minOthers))

class TooFewUniqueCharacters(InvalidPassword):
    __doc__ = _('''Password does not contain enough unique characters.''')

    def __init__(self, minUniqueCharacters=None):
        super(TooFewUniqueCharacters, self).__init__()
        self.minUniqueCharacters = minUniqueCharacters
        if minUniqueCharacters is not None:
            self.i18n_message = _(
                'Password does not contain enough unique characters'
                ' (should have at least ${minUniqueCharacters}).',
                mapping=dict(minUniqueCharacters=minUniqueCharacters))

class TooFewUniqueLetters(InvalidPassword):
    __doc__ = _('''Password does not contain enough unique letters.''')

    def __init__(self, minUniqueLetters=None):
        super(TooFewUniqueLetters, self).__init__()
        self.minUniqueLetters = minUniqueLetters
        if minUniqueLetters is not None:
            self.i18n_message = _(
                'Password does not contain enough unique letters'
                ' (should have at least ${minUniqueLetters}).',
                mapping=dict(minUniqueLetters=minUniqueLetters))

class PasswordExpired(Exception):
    __doc__ = _('''The password has expired.''')

    def __init__(self, principal):
        self.principal = principal
        Exception.__init__(self, self.__doc__)

class PreviousPasswordNotAllowed(InvalidPassword):
    __doc__ = _('''The password set was already used before.''')

    def __init__(self, principal):
        self.principal = principal
        Exception.__init__(self, self.__doc__)

class TooManyLoginFailures(Exception):
    __doc__ = _('''The password was entered incorrectly too often.''')

    def __init__(self, principal):
        self.principal = principal
        Exception.__init__(self, self.__doc__)

TML_CHECK_ALL = None
TML_CHECK_NONRESOURCE = 'nonres'
TML_CHECK_POSTONLY = 'post'

class AccountLocked(Exception):
    __doc__ = _('The account is locked, because the password was '
                'entered incorrectly too often.')

    def __init__(self, principal):
        self.principal = principal
        Exception.__init__(self, self.__doc__)


class IPasswordUtility(zope.interface.Interface):
    """Component to verify and generate passwords.

    The purpose of this utility is to make common password-related tasks, such
    as verification and creation simple. However, only the collection of those
    utilites provide an overall net worth.
    """

    description = zope.schema.Text(
        title=_(u'Description'),
        description=_(u'A description of the password utility.'),
        required=False)

    def verify(new, ref=None):
        """Check whether the new password is valid.

        When a passward is good, the method simply returns, otherwise an
        ``InvalidPassword`` exception is raised.

        It is up to the implementation to define the semantics of a valid
        password. The sematics should ideally be described in the description.

        The ``ref`` argument is a reference password. In many scenarios it
        will be the old password, so that the method can ensure sufficient
        dissimilarity between the new and old password.
        """

    def generate(ref=None):
        """Generate a valid password.

        The ``ref`` argument is a reference password. In many scenarios it
        will be the old password, so that the method can ensure sufficient
        dissimilarity between the new and old password.
        """


class IHighSecurityPasswordUtility(IPasswordUtility):
    """A password utility for very secure passwords."""

    minLength = zope.schema.Int(
        title=_(u'Minimum Length'),
        description=_(u'The minimum length of the password.'),
        required=False,
        default=None)

    maxLength = zope.schema.Int(
        title=_(u'Maximum Length'),
        description=_(u'The maximum length of the password.'),
        required=False,
        default=None)

    @zope.interface.invariant
    def minMaxLength(task):
        if task.minLength is not None and task.maxLength is not None:
            if task.minLength > task.maxLength:
                raise zope.interface.Invalid(
                    u"Minimum length must not be greater than the maximum length.")

    groupMax = zope.schema.Int(
        title=_(u'Maximum Characters of Group'),
        description=_(u'The maximum amount of characters that a password can '
                      u'have from one group. The groups are: digits, letters, '
                      u'punctuation.'),
        required=False,
        default=None)

    maxSimilarity = zope.schema.Float(
        title=_(u'Old/New Similarity'),
        description=(u'The similarity ratio between the new and old password.'),
        required=False,
        default=None)

    minLowerLetter = zope.schema.Int(
        title=_(u'Minimum Number of Lowercase letters'),
        description=_(u'The minimum amount of lowercase letters that a '
                      u'password must have.'),
        required=False,
        default=None)

    minUpperLetter = zope.schema.Int(
        title=_(u'Minimum Number of Uppercase letters'),
        description=_(u'The minimum amount of uppercase letters that a '
                      u'password must have.'),
        required=False,
        default=None)

    minDigits = zope.schema.Int(
        title=_(u'Minimum Number of Numeric digits'),
        description=_(u'The minimum amount of numeric digits that a '
                      u'password must have.'),
        required=False,
        default=None)

    minSpecials = zope.schema.Int(
        title=_(u'Minimum Number of Special characters'),
        description=_(u'The minimum amount of special characters that a '
                      u'password must have.'),
        required=False,
        default=None)

    #WARNING! generating a password with Others is not yet supported
    minOthers = zope.schema.Int(
        title=_(u'Minimum Number of Other characters'),
        description=_(u'The minimum amount of other characters that a '
                      u'password must have.'),
        required=False,
        default=None)

    @zope.interface.invariant
    def saneMinimums(task):
        minl = 0
        if task.minLowerLetter:
            if task.minLowerLetter > task.groupMax:
                raise zope.interface.Invalid(
                    u"Any group minimum length must NOT be greater than "
                    u"the maximum group length.")

            minl += task.minLowerLetter
        if task.minUpperLetter:
            if task.minUpperLetter > task.groupMax:
                raise zope.interface.Invalid(
                    u"Any group minimum length must NOT be greater than "
                    u"the maximum group length.")

            minl += task.minUpperLetter
        if task.minDigits:
            if task.minDigits > task.groupMax:
                raise zope.interface.Invalid(
                    u"Any group minimum length must NOT be greater than "
                    u"the maximum group length.")

            minl += task.minDigits
        if task.minSpecials:
            if task.minSpecials > task.groupMax:
                raise zope.interface.Invalid(
                    u"Any group minimum length must NOT be greater than "
                    u"the maximum group length.")

            minl += task.minSpecials
        if task.minOthers:
            if task.minOthers > task.groupMax:
                raise zope.interface.Invalid(
                    u"Any group minimum length must NOT be greater than "
                    u"the maximum group length.")

            minl += task.minOthers

        if task.maxLength is not None:
            if minl > task.maxLength:
                raise zope.interface.Invalid(
                    u"Sum of group minimum lengths must NOT be greater than "
                    u"the maximum password length.")

    minUniqueLetters = zope.schema.Int(
        title=_(u'Minimum Number of Unique letters'),
        description=_(u'The minimum amount of unique letters that a '
                      u'password must have. This is against passwords '
                      u'like `aAaA0000`. All characters taken lowercase.'),
        required=False,
        default=None)

    @zope.interface.invariant
    def minUniqueLettersLength(task):
        if (task.minUniqueLetters is not None
            and task.minUniqueLetters is not None):
            if task.minUniqueLetters > task.maxLength:
                raise zope.interface.Invalid(
                    u"Minimum unique letters number must not be greater than "
                    u"the maximum length.")

    minUniqueCharacters = zope.schema.Int(
        title=_(u'Minimum Number of Unique characters'),
        description=_(u'The minimum amount of unique characters that a '
                      u'password must have. This is against passwords '
                      u'like `aAaA0000`. All characters taken lowercase.'),
        required=False,
        default=None)

    @zope.interface.invariant
    def minUniqueCharactersLength(task):
        if (task.minUniqueCharacters is not None
            and task.minUniqueCharacters is not None):
            if task.minUniqueCharacters > task.maxLength:
                raise zope.interface.Invalid(
                    u"Minimum unique characters length must not be greater than "
                    u"the maximum length.")



class IPasswordOptionsUtility(zope.interface.Interface):
    """Different general security options.

    The purpose of this utility is to make common password-related options
    available
    """

    changePasswordOnNextLogin = zope.schema.Bool(
        title=_(u'Password must be changed on next login'),
        description=_(u'Password must be changed on next login'),
        required=False,
        default=False)

    passwordExpiresAfter = zope.schema.Int(
        title=_(u'Password expires after (days)'),
        description=_(u'Password expires after (days)'),
        required=False,
        default=None)

    lockOutPeriod = zope.schema.Int(
        title=_(u'Lockout period (minutes)'),
        description=_(u'Lockout the user after too many failed password entries'
                       'for this many minutes. The user can try again after.'),
        required=False,
        default=None)

    maxFailedAttempts = zope.schema.Int(
        title=_(u'Max. number of failed password entries before account is locked'),
        description=_(u'Specifies the amount of failed attempts allowed to check '
                      'the password before the password is locked and no new '
                      'password can be provided.'),
        required=False,
        default=None)

    failedAttemptCheck = zope.schema.Choice(
        title=_(u'Failed password check method'),
        description=_(u'Failed password check method. '
                      'All requests, non-reqource requests, POST requests.'),
        required=False,
        values=[TML_CHECK_ALL, TML_CHECK_NONRESOURCE, TML_CHECK_POSTONLY],
        default=TML_CHECK_ALL )

    disallowPasswordReuse = zope.schema.Bool(
        title=_(u'Disallow Password Reuse'),
        description=_(u'Do not allow to set a previously set password again.'),
        required=False,
        default=False)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Password Utility Implementation
"""
import difflib
import random
import string
import time
import zope.interface
from zope.schema.fieldproperty import FieldProperty

from z3c.password import interfaces


@zope.interface.implementer(interfaces.IPasswordUtility)
class TrivialPasswordUtility(object):
    """A trivial password utility."""

    description = (u'All passwords are accepted and always the "trivial" '
                   u'password is generated.')

    def verify(self, new, ref=None):
        '''See interfaces.IPasswordUtility'''
        return

    def generate(self, ref=None):
        '''See interfaces.IPasswordUtility'''
        return 'trivial'


@zope.interface.implementer(interfaces.IHighSecurityPasswordUtility)
class HighSecurityPasswordUtility(object):
    """An implementation of the high-security password API."""

    minLength = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minLength'])
    maxLength = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['maxLength'])
    groupMax = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['groupMax'])
    maxSimilarity = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['maxSimilarity'])
    minLowerLetter = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minLowerLetter'])
    minUpperLetter = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minUpperLetter'])
    minDigits = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minDigits'])
    minSpecials = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minSpecials'])
    minOthers = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minOthers'])
    minUniqueCharacters = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minUniqueCharacters'])
    minUniqueLetters = FieldProperty(
        interfaces.IHighSecurityPasswordUtility['minUniqueLetters'])

    LOWERLETTERS = string.ascii_letters[:26]
    UPPERLETTERS = string.ascii_letters[26:]
    DIGITS = string.digits
    SPECIALS = string.punctuation

    description = (u'Passwords generated and verified by this utility conform '
                   u'strictly to the specified parameters. See the interface '
                   u'for more details.')

    def __init__(self, minLength=8, maxLength=12, groupMax=6,
                 maxSimilarity=0.6, seed=None,
                 minLowerLetter=None, minUpperLetter=None, minDigits=None,
                 minSpecials=None, minOthers=None,
                 minUniqueCharacters=None, minUniqueLetters=None):
        self.minLength = minLength
        self.maxLength = maxLength
        self.groupMax = groupMax
        self.maxSimilarity = maxSimilarity
        self.random = random.Random(seed or time.time())
        self.minLowerLetter = minLowerLetter
        self.minUpperLetter = minUpperLetter
        self.minDigits = minDigits
        self.minSpecials = minSpecials
        self.minOthers = minOthers
        self.minUniqueCharacters = minUniqueCharacters
        self.minUniqueLetters = minUniqueLetters

    def _checkSimilarity(self, new, ref):
        similarity = difflib.SequenceMatcher(None, new, ref).ratio()
        if similarity > self.maxSimilarity:
            raise interfaces.TooSimilarPassword(
                similarity=similarity, maxSimilarity=self.maxSimilarity)

    def verify(self, new, ref=None):
        '''See interfaces.IHighSecurityPasswordUtility'''
        # 0. Make sure we got a password.
        if not new:
            raise interfaces.NoPassword()
        # 1. Make sure the password has the right length.
        if len(new) < self.minLength:
            raise interfaces.TooShortPassword(minLength=self.minLength)
        if len(new) > self.maxLength:
            raise interfaces.TooLongPassword(maxLength=self.maxLength)
        # 2. Ensure that the password is sufficiently different to the old
        #    one.
        if ref is not None:
            self._checkSimilarity(new, ref)
        # 3. Ensure that the password's character set is complex enough.
        num_lower_letters = 0
        num_upper_letters = 0
        num_digits = 0
        num_specials = 0
        num_others = 0
        uniqueChars = set()
        uniqueLetters = set()
        for char in new:
            uniqueChars.add(char.lower())
            if char in self.LOWERLETTERS:
                num_lower_letters += 1
                uniqueLetters.add(char.lower())
            elif char in self.UPPERLETTERS:
                num_upper_letters += 1
                uniqueLetters.add(char.lower())
            elif char in self.DIGITS:
                num_digits += 1
            elif char in self.SPECIALS:
                num_specials += 1
            else:
                num_others += 1
        if (num_lower_letters > self.groupMax or
            num_upper_letters > self.groupMax or
            num_digits > self.groupMax or
            num_specials > self.groupMax or
            num_others > self.groupMax):
            raise interfaces.TooManyGroupCharacters(
                groupMax=self.groupMax)

        if (self.minLowerLetter is not None
            and num_lower_letters < self.minLowerLetter):
            raise interfaces.TooFewGroupCharactersLowerLetter(
                        minLowerLetter=self.minLowerLetter)

        if (self.minUpperLetter is not None
            and num_upper_letters < self.minUpperLetter):
            raise interfaces.TooFewGroupCharactersUpperLetter(
                        minUpperLetter=self.minUpperLetter)

        if (self.minDigits is not None
            and num_digits < self.minDigits):
            raise interfaces.TooFewGroupCharactersDigits(
                        minDigits=self.minDigits)

        if (self.minSpecials is not None
            and num_specials < self.minSpecials):
            raise interfaces.TooFewGroupCharactersSpecials(
                        minSpecials=self.minSpecials)

        if (self.minOthers is not None
            and num_others < self.minOthers):
            raise interfaces.TooFewGroupCharactersOthers(
                        minOthers=self.minOthers)

        if (self.minUniqueCharacters is not None
            and len(uniqueChars) < self.minUniqueCharacters):
            raise interfaces.TooFewUniqueCharacters(
                        minUniqueCharacters=self.minUniqueCharacters)

        if (self.minUniqueLetters is not None
            and len(uniqueLetters) < self.minUniqueLetters):
            raise interfaces.TooFewUniqueLetters(
                        minUniqueLetters=self.minUniqueLetters)

        return

    def generate(self, ref=None):
        '''See interfaces.IHighSecurityPasswordUtility'''
        verified = False
        while not verified:
            new = ''
            # Determine the length of the password
            length = self.random.randint(self.minLength, self.maxLength)

            # Generate the password
            chars = self.LOWERLETTERS + self.UPPERLETTERS + \
                    self.DIGITS + self.SPECIALS

            for count in range(length):
                new += self.random.choice(chars)

            # Verify the new password
            try:
                self.verify(new, ref)
            except interfaces.InvalidPassword:
                verified = False
            else:
                verified = True
        return new

@zope.interface.implementer(interfaces.IPasswordOptionsUtility)
class PasswordOptionsUtility(object):
    """An implementation of the security options."""

    changePasswordOnNextLogin = FieldProperty(
        interfaces.IPasswordOptionsUtility['changePasswordOnNextLogin'])
    passwordExpiresAfter = FieldProperty(
        interfaces.IPasswordOptionsUtility['passwordExpiresAfter'])
    lockOutPeriod = FieldProperty(
        interfaces.IPasswordOptionsUtility['lockOutPeriod'])
    maxFailedAttempts = FieldProperty(
        interfaces.IPasswordOptionsUtility['maxFailedAttempts'])
    disallowPasswordReuse = FieldProperty(
        interfaces.IPasswordOptionsUtility['disallowPasswordReuse'])
    failedAttemptCheck = FieldProperty(
        interfaces.IPasswordOptionsUtility['failedAttemptCheck'])

    def __init__(self, changePasswordOnNextLogin=None,
                 passwordExpiresAfter=None,
                 lockOutPeriod=None, maxFailedAttempts=None,
                 disallowPasswordReuse=None,
                 failedAttemptCheck=None):
        self.changePasswordOnNextLogin = changePasswordOnNextLogin
        self.passwordExpiresAfter = passwordExpiresAfter
        self.lockOutPeriod = lockOutPeriod
        self.maxFailedAttempts = maxFailedAttempts
        self.disallowPasswordReuse = disallowPasswordReuse
        self.failedAttemptCheck = failedAttemptCheck



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test Setup
"""
import doctest
import re
import unittest
from doctest import DocFileSuite
from zope.testing import renormalizing

from z3c.password import testing

checker = renormalizing.RENormalizing([
    # Python 3 bytes add a "b".
    (re.compile("b('.*?')"),
     r"\1"),
    (re.compile('b(".*?")'),
     r"\1"),
    # Python 3 adds module name to exceptions.
    (re.compile("z3c.password.interfaces.NoPassword"),
     r"NoPassword"),
    (re.compile("zope.security.interfaces.NoInteraction"),
     r"NoInteraction"),
    ])


def test_suite():
    flags = doctest.NORMALIZE_WHITESPACE|\
            doctest.ELLIPSIS|\
            doctest.IGNORE_EXCEPTION_DETAIL
    return unittest.TestSuite((
        DocFileSuite('README.txt',
                     setUp=testing.setUp, tearDown=testing.tearDown,
                     optionflags=flags, checker=checker,
                     ),
        DocFileSuite('principal.txt',
                     setUp=testing.setUp, tearDown=testing.tearDown,
                     optionflags=flags, checker=checker,
                     ),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup
"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup (
    name='z3c.password',
    version='1.0.0',
    author = "Stephan Richter, Roger Ineichen and the Zope Community",
    author_email = "zope3-dev@zope.org",
    description = "Password generation and verification utility for Zope3",
    long_description=(
        read('README.rst')
        + '\n\n' +
        read('CHANGES.rst')
        ),
    license = "ZPL 2.1",
    keywords = "zope3 z3c password verification",
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        "Programming Language :: Python :: Implementation :: PyPy",
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope :: 3'],
    url = 'http://pypi.org/project/z3c.password',
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['z3c'],
    extras_require = dict(
        test = [
            'z3c.coverage',
            'zope.password',
            'zope.pluggableauth',
            'zope.testing',
            ],
        ),
    install_requires = [
        'setuptools',
        'zope.component',
        'zope.exceptions',
        'zope.i18nmessageid',
        'zope.i18n',
        'zope.interface',
        'zope.schema',
        'zope.security',
        ],
    tests_require = [
        'zope.password',
        'zope.pluggableauth',
        'zope.testing',
        ],
    test_suite='z3c.password.tests.test_suite',
    zip_safe = False,
)
